@@ -1,6 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from urlparse import urljoin
+from urllib.parse import urljoin
 from zeep import Client
 from zeep.transports import Transport
 
